@@ -1,12 +1,10 @@
 ### oscorespls.fit.R: The multiresponse orthogonal scores algorithm
 ###
-### $Id: oscorespls.fit.R 133 2007-08-24 09:21:56Z bhm $
-###
 ### Implements an adapted version of the `orthogonal scores' algorithm as
 ###   described in Martens and Naes, pp. 121--122 and 157--158.
 
-oscorespls.fit <- function(X, Y, ncomp, stripped = FALSE,
-                           tol = .Machine$double.eps^0.5, ...)
+oscorespls.fit <- function(X, Y, ncomp, center = TRUE, stripped = FALSE,
+                           tol = .Machine$double.eps^0.5, maxit = 100, ...)
 {
     ## Initialise
     Y <- as.matrix(Y)
@@ -19,7 +17,7 @@ oscorespls.fit <- function(X, Y, ncomp, stripped = FALSE,
     ## as far as it has any effect, it hurts a tiny bit in most situations.
     dimnames(X) <- dimnames(Y) <- NULL
 
-    nobj <- dim(X)[1]
+    nobj  <- dim(X)[1]
     npred <- dim(X)[2]
     nresp <- dim(Y)[2]
 
@@ -33,15 +31,22 @@ oscorespls.fit <- function(X, Y, ncomp, stripped = FALSE,
     }
 
     ## C1
-    Xmeans <- colMeans(X)
-    X <- X - rep(Xmeans, each = nobj)
-    Ymeans <- colMeans(Y)
-    Y <- Y - rep(Ymeans, each = nobj)
+    if (center) {
+        Xmeans <- colMeans(X)
+        X <- X - rep(Xmeans, each = nobj)
+        Ymeans <- colMeans(Y)
+        Y <- Y - rep(Ymeans, each = nobj)
+    } else {
+        ## Set means to zero. Will ensure that predictions do not take the
+        ## mean into account.
+        Xmeans <- rep_len(0, npred)
+        Ymeans <- rep_len(0, nresp)
+    }
 
     ## Must be done here due to the deflation of X
     if (!stripped) Xtotvar <- sum(X * X)
 
-    for(a in 1:ncomp) {
+    for (a in 1:ncomp) {
         ## Initial values:
         if (nresp == 1) {               # pls1
             u.a <- Y                    # FIXME: scale?
@@ -50,7 +55,9 @@ oscorespls.fit <- function(X, Y, ncomp, stripped = FALSE,
             u.a <- Y[,which.max(colSums(Y * Y))]
             t.a.old <- 0
         }
+        nit <- 0
         repeat {
+            nit <- nit + 1              # count the iterations
             ## C2.1
             w.a <- crossprod(X, u.a)
             w.a <- w.a / sqrt(c(crossprod(w.a)))
@@ -76,6 +83,10 @@ oscorespls.fit <- function(X, Y, ncomp, stripped = FALSE,
                 u.a <- Y %*% q.a / c(crossprod(q.a))
                 t.a.old <- t.a          # Save for comparison
             }
+            if (nit >= maxit) {
+              warning("No convergence in ", maxit, " iterations\n")
+              break
+            }
         }
 
         ## C2.3 contd.
@@ -86,12 +97,12 @@ oscorespls.fit <- function(X, Y, ncomp, stripped = FALSE,
         Y <- Y - t.a %*% t(q.a)
 
         ## Save scores etc:
-        W[,a] <- w.a
-        P[,a] <- p.a
+        W[,a]  <- w.a
+        P[,a]  <- p.a
         tQ[a,] <- q.a
         if (!stripped) {
             TT[,a] <- t.a
-            U[,a] <- u.a
+            U[,a]  <- u.a
             tsqs[a] <- tsq
             ## (For very tall, slim X and Y, X0 %*% B[,,a] is slightly faster,
             ## due to less overhead.)
