@@ -1,8 +1,6 @@
 ### svdpc.fit.R: SVD PC fit algorithm
-### $Id: svdpc.fit.R 108 2007-03-19 17:46:06Z bhm $
 
-svdpc.fit <- function(X, Y, ncomp, stripped = FALSE, ...)
-{
+svdpc.fit <- function(X, Y, ncomp, center = TRUE, stripped = FALSE, ...) {
     Y <- as.matrix(Y)
     if (!stripped) {
         ## Save dimnames:
@@ -13,7 +11,7 @@ svdpc.fit <- function(X, Y, ncomp, stripped = FALSE, ...)
     ## as far as it has any effect, it hurts a tiny bit in most situations).
     ## dimnames(X) <- dimnames(Y) <- NULL
 
-    nobj <- dim(X)[1]
+    nobj  <- dim(X)[1]
     npred <- dim(X)[2]
     nresp <- dim(Y)[2]
 
@@ -21,15 +19,22 @@ svdpc.fit <- function(X, Y, ncomp, stripped = FALSE, ...)
     if (!stripped) fitted <- array(0, dim = c(nobj, nresp, ncomp))
 
     ## Center variables:
-    Xmeans <- colMeans(X)
-    X <- X - rep(Xmeans, each = nobj)
-    Ymeans <- colMeans(Y)
-    Y <- Y - rep(Ymeans, each = nobj)
+    if (center) {
+        Xmeans <- colMeans(X)
+        X <- X - rep(Xmeans, each = nobj)
+        Ymeans <- colMeans(Y)
+        Y <- Y - rep(Ymeans, each = nobj)
+    } else {
+        ## Set means to zero. Will ensure that predictions do not take the
+        ## mean into account.
+        Xmeans <- rep_len(0, npred)
+        Ymeans <- rep_len(0, nresp)
+    }
 
     huhn <- La.svd(X)
-    D <- huhn$d[1:ncomp]
+    D  <- huhn$d[1:ncomp]
     TT <- huhn$u[,1:ncomp, drop=FALSE] %*% diag(D, nrow = ncomp)
-    P <- t(huhn$vt[1:ncomp,, drop=FALSE])
+    P  <- t(huhn$vt[1:ncomp,, drop=FALSE])
     tQ <- crossprod(TT, Y) / D^2
 
     for (a in 1:ncomp) {
